#!/usr/bin/perl

use strict;
use Data::Dumper::Simple;
require("libmreg.pl");
use DBI;

###############################################################################
## C O N F I G U R A T I O N   S E C T I O N                                  #
###############################################################################

## Edit these values for login to the API
my $api_login = "LOGIN";
my $api_password = "PASSWORD";

## subusers of your main login whose subusers are synchronized to OTRS database
my @api_users = ("subuser1.tld", "subuser2.tld");

## Edit these values for login to your OTRS-database
my $db_name = "otrs";
my $db_host = "DBHOST";
my $db_dsn = "DBI:mysql:database=$db_name;host=$db_host";
my $db_user = "otrs";
my $db_password = "OTRSPASSWD";

## Uncomment these lines to add all API user email addresses to your database (e.g. for white listing on your mail exchange server)
#my $db_mail_name = "mail";
#my $db_mail_host = "DBHOST";
#my $db_mail_dsn = "DBI:mysql:database=$db_mail_name;host=$db_mail_host";;
#my $db_mail_user = "mail";
#my $db_mail_password = "MAILPASSWD";

# Add your API-IP or -Hostname
my $xhost = "93.190.235.100";


###############################################################################
## P R O G R A M M   S E C T I O N                                            #
## please do not edit beyond this line                                        #
###############################################################################

# init db
my $dbh = DBI->connect($db_dsn, $db_user, $db_password, {RaiseError => 1});
my $dbh_mail;
if ($db_mail_dsn) {
	$dbh_mail = DBI->connect($db_mail_dsn, $db_mail_user, $db_mail_password, {RaiseError => 0, PrintError => 0});
}


my $c_ins = 0;
my $c_upd = 0;
my @user_emails;
foreach my $api_user (@api_users) {
	# query user list
	my $config = { 'socket' => 'http://' . $xhost . '/api/call.cgi?s_login=' . $api_login . '&s_pw=' . $api_password . '&s_user=' . $api_user };
#	debug(Dumper($config));

	my $command = { 'command' => 'QueryUserList', "userdepth" => 'SUBUSER'};
	my $response = mreg_call($command, $config);
	if ($$response{CODE} != 200) {
		printError("ERROR: QueryUserList: code: " . $$response{CODE} . " - description: " . $$response{DESCRIPTION});
		exit(1);
	}
	#debug(Dumper($response));
	my $property = $$response{"PROPERTY"};
	my @userList = @{$$property{USER}};

	# query user environment
	foreach my $user (@userList) {
		debug("user: " . $user);
	
		my $email2 = "";
	
		# query user environment
		my $config = { 'socket' => 'http://' . $xhost . '/api/call.cgi?s_login=' . $api_login . '&s_pw=' . $api_password . '&s_user=' . $user };
#		debug(Dumper($config));
		my $command = { 'command' => 'QueryEnvironmentList', 'environmentkey' => 'user-info/contact/company'};
		my $response = mreg_call($command, $config);
		if ($$response{CODE} != 200) {
			printError("ERROR: code: " . $$response{CODE} . " - description: " . $$response{DESCRIPTION});
			next;
		}
#		debug(Dumper($response));
		my $property = $$response{"PROPERTY"};
		if (!$property) {
			next;
		}
		my @envNames = @{$$property{ENVIRONMENTNAME}};
		my @envValues = @{$$property{ENVIRONMENTVALUE}};

		my %envData;
		my $counter = 0;
		foreach my $envName (@envNames) {
			$envData{$envName} = $envValues[$counter];
			$counter++;
		}
#		debug(Dumper(%envData));

		if (!($envData{email} =~ /@/)) {
			printError("  -> ERROR: bad email address: " . $envData{email});
			next;
		}
		if ($envData{email2} =~ /@/) {
			$email2 = $envData{email2};
		}
		if (!($envData{title})) {
			$envData{title} = "";
		}
		if (!($envData{firstname})) {
			printError("  -> ERROR: no firstname");
			next;
		}
		if (!($envData{lastname})) {
			printError("  -> ERROR: no lastname");
			next;
		}
		if (!($envData{organization})) {
			$envData{organization} = "";
		}
		if (!($envData{phone})) {
			$envData{phone} = "";
		}
		if (!($envData{country})) {
			$envData{country} = "";
		}
		my $otrs_lang = "en";
		if ($envData{country} eq "DE") {
			$otrs_lang = "de";
		}


		push(@user_emails, $envData{email});
		if ($email2) {
#			print "secondary email: $email2\n";
			push(@user_emails, $email2);
		}

		# generate random password (like session id)
		my $rpwd = int(rand()*time()) . int(rand()*time());
		

		# check if user preference record exist
		my $query = "SELECT user_id FROM customer_preferences WHERE (user_id = " . $dbh->quote($user) . ") AND (preferences_key = 'UserLanguage')";
		my $sth = $dbh->prepare($query);
		$sth->execute();
		if (my @row = $sth->fetchrow_array()) {
			$query = "UPDATE customer_preferences SET ";
			$query .= "preferences_value = " . $dbh->quote($otrs_lang);
			$query .= " WHERE (user_id = " . $dbh->quote($user) . ") AND (preferences_key = 'UserLanguage')";
			debug("  -> query: $query");
			$dbh->do($query);
		}
		else {
			$query = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES (";
			$query .= $dbh->quote($user) . ", ";
			$query .= $dbh->quote("UserLanguage") . ", ";
			$query .= $dbh->quote($otrs_lang);
			$query .= ")";
			debug("  -> query: $query");
			$dbh->do($query);
		}


		# check if user exists in OTRS
		my $query = "SELECT customer_id FROM customer_user WHERE (customer_id = " . $dbh->quote($user) . ")";
		my $sth = $dbh->prepare($query);
		$sth->execute();
		if (my @row = $sth->fetchrow_array()) {
			# user already exists
			debug("  -> user already exists");
			
			$query = "UPDATE customer_user SET ";
			$query .= "login = " . $dbh->quote($user) . ", ";
			$query .= "email = " . $dbh->quote($envData{email}) . ", ";
			$query .= "pw = " . $dbh->quote($rpwd) . ", ";
			$query .= "salutation = " . $dbh->quote($envData{title}) . ", ";
			$query .= "first_name = " . $dbh->quote($envData{firstname}) . ", ";
			$query .= "last_name = " . $dbh->quote($envData{lastname}) . ", ";
			$query .= "organization = " . $dbh->quote($envData{organization}) . ", ";
			$query .= "phone = " . $dbh->quote($envData{phone}) . ", ";
			$query .= "comments = " . $dbh->quote("") . ", ";
			$query .= "valid_id = " . "1" . "";
			$query .= " WHERE (customer_id = " . $dbh->quote($user) . ")";
			debug("  -> query: $query");
			$dbh->do($query);
			$c_upd++;

			next;
		}
	

		# insert new database record
		$query = "INSERT INTO customer_user (login, email, customer_id, pw, salutation, first_name, last_name, organization, phone, comments, valid_id) VALUES (";
		$query .= $dbh->quote($user) . ", ";
		$query .= $dbh->quote($envData{email}) . ", ";
		$query .= $dbh->quote($user) . ", ";
		$query .= $dbh->quote($rpwd) . ", ";
		$query .= $dbh->quote($envData{title}) . ", ";
		$query .= $dbh->quote($envData{firstname}) . ", ";
		$query .= $dbh->quote($envData{lastname}) . ", ";
		$query .= $dbh->quote($envData{organization}) . ", ";
		$query .= $dbh->quote($envData{phone}) . ", ";
		$query .= $dbh->quote("") . ", ";
		$query .= "1" . "";
		$query .= ")";
		debug("  -> query: $query");
		$dbh->do($query);
		$c_ins++;
	}
}
debug("queries: INSERT: $c_ins / UPDATE: $c_upd");

if ($db_mail_dsn) {
	foreach my $user_email (@user_emails) {
			my $query = "INSERT INTO support_customer_email (email) VALUES (";
			$query .= $dbh->quote($user_email);
			$query .= ")";
			debug("  -> query: $query");
			$dbh_mail->do($query);
	}
}


# subs
sub debug
{
	my ($msg) = @_;
#	print $msg . "\n";
}

sub printError
{
	my ($msg) = @_;
	print $msg . "\n";
}
